#!/bin/sh

PATH="$PATH:/usr/bin:/bin:/usr/sbin:/sbin"
export PATH

AGENTSRV=swiagentd
AGENTDIR=/opt/SolarWinds/Agent
AGENTROOT=$AGENTDIR/bin
AGENTBIN=$AGENTROOT/swiagent

showhelp()
{
	cat << ENDOFBLOCK
Allows doing specific actions for the agent.
Assumed to be called from the agent process.
May specify multiple actions one after another.

Supported actions:
swiagentaid.sh <service-action> <service-name>
where service-action is one of enable/disable/status/start/stop/restart
  returns or modifies the state of the service given by service-name
swiagentaid.sh init <variable parameters>
  calls the agent passing given arguments
  No further actions can be specified after this action! All remaining
  parameters are treated as parameters for init.
swiagentaid.sh permissions
  Ensures files and directories under the agent tree have the right permissions.
swiagentaid.sh sleep ms
  waits for given amount of ms before continuing
swiagentaid.sh echo text
  prints what is in the next parameter
swiagentaid.sh help
  prints this help and exits
ENDOFBLOCK
}

# determines whether a executable program with name given by first parameter
# exists in the system
canrun()
{
	which "$1" > /dev/null 2> /dev/null
}

# wait given amount of milliseconds
# $1 .. number of ms to sleep
waitms()
{
	$AGENTBIN /sleep="$1"
}

# enables or disables the swiagentd specific cronjob
# $1 .. "enable"/"disable"
modifycron()
{
	if [ "$1" = enable ]; then
		(crontab -l | grep -v "$AGENTSRV" ; echo "* * * * * /bin/sh /etc/init.d/$AGENTSRV swrestart > /dev/null 2>&1") | crontab -
	else
		crontab -l | grep -v "$AGENTSRV" | crontab -
	fi
}

# verify if it is possible to mofify the service using initctl
isupstartenabled()
{
	canrun initctl || return 1
	srvname="$1"
	case "$srvname" in
		swiagentd) srvname=swiagent ;;
		swiagentd.update) srvname=swiagent.update ;;
	esac
	[ -r /etc/init/"$srvname".conf ]
}

# call's the system local command to enable/disable service
# $1 .. "enable"/"disable"
# $2 .. service name
modifyservice()
{
	cronjob=0
	result=0
	servicemgr=$(system_service_manager)

	case "$servicemgr" in
		upstart)
			srvname="$2"
			case "$srvname" in
				swiagentd) srvname=swiagent ;;
				swiagentd.update) srvname=swiagent.update ;;
			esac
			# override upstart .override file to enable/disable service
			if [ "$1" = enable ]; then
				rm -f /etc/init/"$srvname".override
			else
				echo manual > /etc/init/"$srvname".override
			fi
			result=$?
			[ $result -eq 0 ] || return $result
			initctl reload-configuration
		;;
		systemd)
			systemctl "$1" "$2".service
			result=$?
		;;
		sysvinit)
			if canrun update-rc.d; then
				if [ "$1" = enable ]; then
					update-rc.d "$2" defaults
				else
					update-rc.d "$2" remove
				fi
				result=$?
				[ "$2" = $AGENTSRV ] && cronjob=1
			elif canrun chkconfig; then
				if [ "$1" = enable ]; then
					chkconfig --add "$2"
				else
					chkconfig --del "$2"
				fi
				result=$?
				[ "$2" = $AGENTSRV ] && cronjob=1
			else
				# no Ubutnu update-rc.d and chkconfig, create manually rc.d symlinks to enable SysV rc.d startup
				[ "$2" = $AGENTSRV ] && cronjob=1
				if [ "$1" = enable ]; then
					for i in 2 3 4 5; do
						ln -sf /etc/init.d/"$2" /etc/rc.d/rc${i}.d/S90"$2" || result=$?
					done
					for i in 0 1 6; do
						ln -sf /etc/init.d/"$2" /etc/rc.d/rc${i}.d/K10"$2" || result=$?
					done
				else
					rm /etc/rc.d/rc?.d/???"$2"
					result=$?
				fi
			fi
			;;
			*)
				echo "unsupported service manager '$servicemgr'"
				result=2
			;;
		esac

	# generate result
	[ $result -eq 0 ] || return $result
	if [ $cronjob -eq 1 ]; then
		modifycron "$1"
		result=$?
	fi
	return $result
}

# wraps the usual service command and does actions on the service like start/stop/restart
# $1 .. action to be performed
# $2 .. service to be modified
servicecommand()
{
    action="$1"
    service_name="$2"

    shift; shift

    servicemgr=$(system_service_manager)

    case "$servicemgr" in
        sysvinit|upstart)
            service "$service_name" "$action" "$@"
            ;;
        systemd)
            if [ "$action" = 'init' ] ; then
                if canrun 'service' ; then
                    service "$service_name" "$action" "$@"
                else
                    /etc/init.d/"$service_name" "$action" "$@"
                fi
            else
                systemctl "$action" "$service_name"
            fi
            ;;
        *)
            echo "unsupported service manager '$servicemgr'"
            return 2
            ;;
    esac
}

# ensures that all files and directories in the given folder have the right permissions
# won't touch the contents of what is given by second parameter
# $1 .. the dir which shall be updated (and recursively what it contains)
# $2 .. exception for recursive change (will apply change to the item only, not recursively)
ensurepermissions()
{
    (
        cd "$1" || exit $?
        ls -A1 | while read i
        do
            if [ "$i" = "$2" ] ; then
                chown swiagent.swiagent "$i" || exit $?
            else
                chown -R swiagent.swiagent "$i" || exit $?
            fi
        done
    )
}
ensurespecialpermissions()
{
    (
        cd $AGENTDIR/bin/Plugins
        chown -R swiagent.swiagent Discovery JobEngine
        chown swiagent.swiagent Discovery.Agent.plugin JobEngine.Agent.plugin
        chown root.swiagent Discovery/SolarWinds.Agent.Discovery.Plugin
        chmod u+xs Discovery/SolarWinds.Agent.Discovery.Plugin
        chown root.swiagent JobEngine/SolarWinds.Agent.JobEngine.Plugin
        chmod u+xs JobEngine/SolarWinds.Agent.JobEngine.Plugin

        # note: __pycache__ block can be removed after few releases
        find . -name __pycache__ | while read i
        do
            chown -R swiagent.swiagent "$i"
        done
    )
}

#
# Detect the service manager in the system.
# Returns : prints the result (systemd/upstart/sysvinit) to the output.
# Error   : print sysvinit.
#
system_service_manager()
{
    pid1=`ls -la /proc/1/exe`
	lastpar() { eval echo \${$#} ; }
	initsystem=`lastpar $pid1`
    if [ -z ${initsystem##*init*} ]; then
        #/proc/1/exe->/sbin/init
        initsystem=`/sbin/init --version 2>&1 | (grep -q "upstart" && echo upstart || echo sysvinit)`
    fi
    if [ -z ${initsystem##*upstart*} ]; then
        echo upstart
    elif [ -z ${initsystem##*systemd*} ]; then
        echo systemd
    else
        echo sysvinit
    fi
}

#
# main script body starts here
#
res=0
while [ $# -gt 0 ]; do
    case "$1" in
        enable|disable)
            modifyservice "$1" "$2"
            res=$?
            shift ; shift
        ;;
        status|start|stop|restart)
            servicecommand "$1" "$2" > /dev/null 2> /dev/null
            res=$?
            shift ; shift
        ;;
        stopanddisable)
            modifyservice disable "$2"
            servicecommand stop "$2" > /dev/null 2> /dev/null
            shift; shift
        ;;
        init)
            shift
            servicecommand init swiagentd "$@"
            exit "$?"
        ;;
        permissions)
            # ensure permissions for everything except for the Plugins, finally ensure JobEngine and Discovery explicitly (within Plugins)
            chown swiagent.swiagent "$AGENTDIR" && ensurepermissions $AGENTDIR bin && ensurepermissions $AGENTDIR/bin Plugins && ensurespecialpermissions
            res=$?
            shift
        ;;
        servicemgr)
            system_service_manager
            shift
        ;;
        sleep)
            waitms "$2"
            res=$?
            shift ; shift
        ;;
        echo)
            echo "$2"
            res=0
            shift ; shift
        ;;
        help)
            showhelp
            res=0
            break
        ;;
        sw-notify-uninstalled)
            cd $AGENTROOT
            $AGENTBIN /logfile /notify_uninstalled >/dev/null 2>/dev/null &
            $AGENTBIN sleep=5000
            res=0 # we ignore result in this case (during uninstallation)
            shift
            ;;
        *)
            echo "Invalid action '$1' specified." >&2
            exit 1
        ;;
    esac
    [ $res -ne 0 ] && break
done

exit $res
